package nl.kallestruik.dlib.gui

import kotlinx.coroutines.runBlocking
import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.NamedTextColor
import org.bukkit.Bukkit
import org.bukkit.Material
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.inventory.InventoryAction
import org.bukkit.event.inventory.InventoryClickEvent
import org.bukkit.event.inventory.InventoryCloseEvent
import org.bukkit.inventory.InventoryView
import org.bukkit.inventory.ItemStack
import org.bukkit.plugin.Plugin

fun confirmationDialog(player: Player, plugin: Plugin, configure: suspend ConfirmationDialog.() -> Unit) {
    val confirmationDialog = ConfirmationDialog()
    runBlocking {
        confirmationDialog.configure()
    }

    confirmationDialog.display(player, plugin)
}

class ConfirmationDialog: Listener {
    private data class Configuration(
        var onConfirm: suspend () -> Unit = {},
        var onCancel: suspend () -> Unit = {}
    )

    var title: Component = Component.empty()
    var description: Component = Component.empty()

    private val configuration = Configuration()
    private lateinit var plugin: Plugin
    private lateinit var view: InventoryView

    fun onConfirm(onConfirm: suspend () -> Unit) {
        configuration.onConfirm = onConfirm
    }

    fun onCancel(onCancel: suspend () -> Unit) {
        configuration.onCancel = onCancel
    }

    fun display(player: Player, plugin: Plugin) {
        player.closeInventory()
        this.plugin = plugin

        val inventory = Bukkit.createInventory(null, 3 * 9, title)
        view = player.openInventory(inventory) ?: return

        val borderStack = ItemStack(Material.ORANGE_STAINED_GLASS_PANE, 1)
        val borderMeta = borderStack.itemMeta
        borderMeta.displayName(Component.empty())
        borderStack.itemMeta = borderMeta

        // Top border
        for (x in 8 downTo 0) {
            view.setItem(x, borderStack)
        }

        // Middle sides
        view.setItem(9, borderStack)
        view.setItem(17, borderStack)

        // Bottom border
        for (x in 26 downTo 18) {
            view.setItem(x, borderStack)
        }

        // Description item
        val descriptionItem = ItemStack(Material.PAPER, 1)
        val descriptionMeta = descriptionItem.itemMeta
        descriptionMeta.displayName(description)
        descriptionItem.itemMeta = descriptionMeta

        // Confirm item
        val confirmItem = ItemStack(Material.SUNFLOWER, 1)
        val confirmMeta = confirmItem.itemMeta
        confirmMeta.displayName(Component.text("Confirm").color(NamedTextColor.GREEN))
        confirmItem.itemMeta = confirmMeta

        // Cancel item
        val cancelItem = ItemStack(Material.BARRIER, 1)
        val cancelMeta = cancelItem.itemMeta
        cancelMeta.displayName(Component.text("Cancel").color(NamedTextColor.RED))
        cancelItem.itemMeta = cancelMeta

        //  0  1  2  3  4  5  6  7  8
        //  9 10 11 12 13 14 15 16 17
        // 18 19 20 21 22 23 24 25 26

        view.setItem(4, descriptionItem)
        view.setItem(11, confirmItem)
        view.setItem(15, cancelItem)

        plugin.server.pluginManager.registerEvents(this, plugin)
    }

    private fun cleanup() {
        view.close()
        HandlerList.unregisterAll(this)
    }

    /*
    Bukkit event handlers
     */
    @EventHandler
    fun onInventoryClose(event: InventoryCloseEvent) {
        if (event.view != this.view)
            return

        if (event.reason == InventoryCloseEvent.Reason.PLUGIN) {
            return
        }

        runBlocking {
            configuration.onCancel.invoke()
            cleanup()
        }
    }

    @EventHandler
    fun onOperationFinish(event: InventoryClickEvent) {
        if (event.view != view) {
            return
        }

        if (event.action != InventoryAction.PICKUP_ALL) {
            event.isCancelled = true
            return
        }

        val slot = view.convertSlot(event.rawSlot)

        if (slot != 11 && slot != 15) {
            event.isCancelled = true
            return
        }

        event.isCancelled = true

        if (slot == 11) {
            runBlocking {
                configuration.onConfirm()
                cleanup()
            }
        } else {
            runBlocking {
                configuration.onCancel()
                cleanup()
            }
        }
    }
}